#include "gui.h"

using namespace imr::gui;


Gui::Gui()
{
    vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();

    measurement = vtkSmartPointer<vtkPolyData>::New();
    measurement->SetPoints(points);

    points = vtkSmartPointer<vtkPoints>::New();
    position = vtkSmartPointer<vtkPolyData>::New();
    position->SetPoints(points);
    
    points = vtkSmartPointer<vtkPoints>::New();
    kfPosition = vtkSmartPointer<vtkPolyData>::New();
    kfPosition->SetPoints(points);
   

    measurementFilter = vtkSmartPointer<vtkVertexGlyphFilter>::New();
    #if VTK_MAJOR_VERSION <= 5
        measurementFilter->AddInput(measurement);
    #else
        measurementFilter->AddInputData(measurement);
    #endif
    measurementFilter->Update();

    measurementMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
    measurementMapper->SetInputConnection(measurementFilter->GetOutputPort());

    measurementActor = vtkSmartPointer<vtkActor>::New();
    measurementActor->SetMapper(measurementMapper);
    measurementActor->GetProperty()->SetColor(1,0,0);
    measurementActor->GetProperty()->SetPointSize(3);
    
    positionFilter = vtkSmartPointer<vtkVertexGlyphFilter>::New();
    #if VTK_MAJOR_VERSION <= 5
        positionFilter->AddInput(position);
    #else
        positionFilter->AddInputData(position);
    #endif
    positionFilter->Update();

    positionMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
    positionMapper->SetInputConnection(positionFilter->GetOutputPort());

    positionActor = vtkSmartPointer<vtkActor>::New();
    positionActor->SetMapper(positionMapper);
    positionActor->GetProperty()->SetColor(0,1,0);
    positionActor->GetProperty()->SetPointSize(5);

    kfPositionFilter = vtkSmartPointer<vtkVertexGlyphFilter>::New();
    #if VTK_MAJOR_VERSION <= 5
        kfPositionFilter->AddInput(kfPosition);
    #else
        kfPositionFilter->AddInputData(kfPosition);
    #endif
    kfPositionFilter->Update();

    kfPositionMapper = vtkSmartPointer<vtkPolyDataMapper>::New();
    kfPositionMapper->SetInputConnection(kfPositionFilter->GetOutputPort());

    kfPositionActor = vtkSmartPointer<vtkActor>::New();
    kfPositionActor->SetMapper(kfPositionMapper);
    kfPositionActor->GetProperty()->SetColor(0,0,1);
    kfPositionActor->GetProperty()->SetPointSize(5);


    renderer = vtkSmartPointer<vtkRenderer>::New();
    renderWindow = vtkSmartPointer<vtkRenderWindow>::New();
    renderWindow->AddRenderer(renderer);

    renderWindowInteractor = vtkSmartPointer<vtkRenderWindowInteractor>::New();
    renderWindowInteractor->SetRenderWindow(renderWindow);
    
    interactorStyle = vtkSmartPointer<vtkInteractorStyleImage>::New();
    renderWindowInteractor->SetInteractorStyle(interactorStyle);

    renderer = vtkSmartPointer<vtkRenderer>::New();
    renderWindow = vtkSmartPointer<vtkRenderWindow>::New();
    renderWindow->AddRenderer(renderer);

    renderWindowInteractor = vtkSmartPointer<vtkRenderWindowInteractor>::New();
    renderWindowInteractor->SetRenderWindow(renderWindow);
    
    interactorStyle = vtkSmartPointer<vtkInteractorStyleImage>::New();
    renderWindowInteractor->SetInteractorStyle(interactorStyle);

    renderer->AddActor(measurementActor);
    renderer->AddActor(positionActor);
    renderer->AddActor(kfPositionActor);
    renderer->SetBackground(1,1,1);
    renderWindow->Render();
}

void Gui::drawCovariance(float x, float y , float sx, float sy) {
}

void Gui::RawPoints2vtkPoints(const RawPoints &pts, vtkSmartPointer<vtkPoints> vtkPts)
{
    for(int i=0; i<pts.size(); i++) {
        vtkPts->InsertNextPoint(pts[i].x, pts[i].y, .0);
    }
}
void Gui::RawPoints2vtkPoints(const Point &pts, vtkSmartPointer<vtkPoints> vtkPts)
{
        vtkPts->InsertNextPoint(pts.x/100.0, pts.y/100.0, .0);
}
void Gui::startInteractor()
{
    renderWindowInteractor->Start();
}


void Gui::clearPoints()
{
    measurement->SetPoints(vtkSmartPointer<vtkPoints>::New());
    position->SetPoints(vtkSmartPointer<vtkPoints>::New());
    kfPosition->SetPoints(vtkSmartPointer<vtkPoints>::New());
    measurement->Modified();
    position->Modified();
    kfPosition->Modified();
    renderWindow->Render();
}


void Gui::setPoints(Point truth, Point measured, Point kfPos)
{
    vtkSmartPointer<vtkPoints> p = position->GetPoints();
    RawPoints2vtkPoints(truth,p);

    position->SetPoints(p);
    position->Modified();

    vtkSmartPointer<vtkPoints> ip = kfPosition->GetPoints();
    RawPoints2vtkPoints(kfPos,ip);

    kfPosition->SetPoints(ip);
    kfPosition->Modified();



    vtkSmartPointer<vtkPoints> points = measurement->GetPoints();
    RawPoints2vtkPoints(measured,points);

    measurement->SetPoints(points);
    measurement->Modified();
    renderWindow->Render();
}
